//
//  MemoryTrainingViewController.m
//  MemoryTraining
//
//  Created by Jiayao Yu on 3/27/10.
//  Copyright __MyCompanyName__ 2010. All rights reserved.
//

#import "GameViewController.h"
#import "GameView.h"
#import "FlipPanel.h"
#import "MemoryTrainingAppDelegate.h"
#import "Session.h"
@implementation GameViewController
@synthesize coreDataHelper;

/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/



// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
  [super viewDidLoad];
  mainView = (GameView *) [self view];
  [mainView setController:self];
  gameOverAlert = (UIAlertView *)[[UIAlertView alloc]
                                        initWithTitle:@"Game Over"
                                        message:@"Do you want to try again?"
                                        delegate:self cancelButtonTitle:@"OK"
                                        otherButtonTitles:@"Main Menu", nil];
  levelUpAlert = (UIAlertView *)[[UIAlertView alloc]
                                        initWithTitle:@"Level Up"
                                        message:@"Congratulations!"
                                        delegate:self cancelButtonTitle:@"OK"
                                        otherButtonTitles:nil];
  level = [[Level alloc] initWithLevel:1];
  [self restart];
}

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
  [level release];
  [gameOverAlert release];
  [levelUpAlert release];
}

- (void)gameOver {
  state = kGameOver;
  [[mainView statusLabel] setText:@"Game Over"];
  [mainView showPattern];
  [mainView setFlipPanelsEnabled:NO];
  NSManagedObjectContext *context = [[coreDataHelper fetchedResultsController] managedObjectContext];
  Session *session = [NSEntityDescription insertNewObjectForEntityForName:@"Session" inManagedObjectContext:context];
  [session setLevel:[NSNumber numberWithInt:level.level]];
  [session setFinished:[NSDate date]];
  [session setGame:@"MemoryMatrix"];
  NSError *error = nil; 
  if (![context save:&error]) {
    NSLog(@"Error occurred when saving session:%d", [error code]);
  }
}

- (IBAction)mainMenu {
  [[self parentViewController] dismissModalViewControllerAnimated:YES];
  [level initWithLevel:1];
}

- (void)touched {
  if (state == kInProgress) {
    NSArray *panels = [mainView panels];
    BOOL finished = YES; // finished means game over or discovered all panels
    for (int i = 0; i < [level size] *[level size]; i++) {
      FlipPanel *panel = [panels objectAtIndex:i];
      if ([panel flipped]) {
        if (![[[level pattern] objectAtIndex:i] boolValue]) {
          [panel setBackgroundColor:[UIColor blackColor]];
          // Show user what she should click instead
          [self gameOver];
        }
      } else if ([[[level pattern] objectAtIndex:i] boolValue]) {
        finished = NO;
      }
    }
    
    if (finished) {
      [levelUpAlert show];
    }
  }
}

- (IBAction)restart {
  state = kInProgress;
  [[mainView statusLabel] setText:@""];
  [level reset];
  [mainView setLevel:level];
  [levelLabel setText:[NSString stringWithFormat:@"Level %d", [level level]]];
  [mainView refresh];
}

- (void)dealloc {
  [super dealloc];
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
  if (alertView == levelUpAlert) {
    levelPassed = level.level;
    [level advance];
    [levelLabel setText:[NSString stringWithFormat:@"Level %d", [level level]]];
    [mainView refresh];
  } else if(alertView == gameOverAlert) {
    if (buttonIndex == 0) {
      [self restart];
    } else if (buttonIndex == 1) {
      [[self parentViewController] dismissModalViewControllerAnimated:YES];
      [level initWithLevel:1];
    }
  }
}

-(void)setLevel:(int)aLevel {
  levelPassed = 0;
  level.level = aLevel;
}
  
@end
