//
//  CPTestAppScatterPlotController.m
//  CPTestApp-iPhone
//
//  Created by Brad Larson on 5/11/2009.
//

#import "StatsViewController.h"
#import "MemoryTrainingAppDelegate.h"
#import "Session.h"
@implementation StatsViewController

@synthesize dataForPlot;
@synthesize coreDataHelper;
#pragma mark -
#pragma mark Initialization and teardown

-(void)dealloc 
{
  [statsView release];
	[dataForPlot release];
  [super dealloc];
}

-(void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
  NSError *error = nil;
  BOOL success = [[coreDataHelper fetchedResultsController] performFetch:&error];
  if (!success) {
    NSLog(@"error occurred %d", [error code]);
  }  
  // Add some initial data
	NSMutableArray *contentArray = [NSMutableArray arrayWithCapacity:100];
  NSArray * results = [[coreDataHelper fetchedResultsController] fetchedObjects];
  int maxLevel = 1;
  for (int i = 0; i < [results count]; i++) {
    Session *session = [results objectAtIndex:i];
    if ([session.level intValue] > maxLevel) {
      maxLevel = [session.level intValue];
    }
    [contentArray addObject:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:i], @"x", [session.level retain], @"y", nil]];
  }
  self.dataForPlot = contentArray;
  
  // Setup scatter plot space
  CPXYPlotSpace *plotSpace = (CPXYPlotSpace *)graph.defaultPlotSpace;
  plotSpace.xRange = [CPPlotRange plotRangeWithLocation:CPDecimalFromFloat(0) length:CPDecimalFromInteger([contentArray count])];
  plotSpace.yRange = [CPPlotRange plotRangeWithLocation:CPDecimalFromInteger(0) length:CPDecimalFromInteger(maxLevel)];
  [graph reloadData];
}

-(void)viewDidLoad {
  [super viewDidLoad];

 
  // Create graph from theme
  graph = [(CPXYGraph *)[CPXYGraph alloc] initWithFrame:CGRectZero];
	CPTheme *theme = [CPTheme themeNamed:kCPDarkGradientTheme];
	[graph applyTheme:theme];
  statsView = [[StatsView alloc] initWithFrame:CGRectMake(0, 0, 320, 400)];
  [self.view addSubview:statsView];
	statsView.hostedLayer = graph;
  
  // Axes
	CPXYAxisSet *axisSet = (CPXYAxisSet *)graph.axisSet;
  CPXYAxis *x = axisSet.xAxis;
  x.majorIntervalLength = CPDecimalFromFloat(1.0);
  x.orthogonalCoordinateDecimal = CPDecimalFromInteger(1);
  x.minorTicksPerInterval = 0;
  NSNumberFormatter *numberFormatter = [[[NSNumberFormatter alloc] init] autorelease];
  [numberFormatter setMaximumFractionDigits:0];
  x.labelFormatter = numberFormatter;
  CPXYAxis *y = axisSet.yAxis;
  y.majorIntervalLength = CPDecimalFromInteger(1);
  y.minorTicksPerInterval = 0;
  y.orthogonalCoordinateDecimal = CPDecimalFromInteger(1);
  y.labelFormatter = numberFormatter;
  
  // Create a plot that uses the data source method
	CPScatterPlot *dataSourceLinePlot = [[[CPScatterPlot alloc] init] autorelease];
  dataSourceLinePlot.identifier = @"Date Plot";
	dataSourceLinePlot.dataLineStyle.lineWidth = 3.f;
  dataSourceLinePlot.dataLineStyle.lineColor = [CPColor greenColor];
  dataSourceLinePlot.dataSource = self;
  [graph addPlot:dataSourceLinePlot];
}

-(void)changePlotRange 
{
  CPXYPlotSpace *plotSpace = (CPXYPlotSpace *)graph.defaultPlotSpace;
  plotSpace.xRange = [CPPlotRange plotRangeWithLocation:CPDecimalFromFloat(0.0) length:CPDecimalFromFloat(3.0 + 2.0*rand()/RAND_MAX)];
  plotSpace.yRange = [CPPlotRange plotRangeWithLocation:CPDecimalFromFloat(0.0) length:CPDecimalFromFloat(3.0 + 2.0*rand()/RAND_MAX)];
}

#pragma mark -
#pragma mark Plot Data Source Methods

-(NSUInteger)numberOfRecordsForPlot:(CPPlot *)plot {
  return [dataForPlot count];
}

-(NSNumber *)numberForPlot:(CPPlot *)plot field:(NSUInteger)fieldEnum recordIndex:(NSUInteger)index 
{
  return [[dataForPlot objectAtIndex:index] valueForKey:(fieldEnum == CPScatterPlotFieldX ? @"x" : @"y")];
}

- (IBAction) mainMenu {
  [[self parentViewController] dismissModalViewControllerAnimated:YES];
}
@end
